# sb::App
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App;
use strict;
use sb ();
use sb::Template ();
use sb::File ();
use sb::Time ();
use sb::Lock ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.13';
# 0.13 [2008/12/01] modified style_path to output correct path
# 0.12 [2008/07/16] fixed static_path to use srv_temp
# 0.11 [2007/12/03] modified initialize to setup sb::Lock at start-up
# 0.10 [2007/09/20] removed string no longer needed, modified load_template and template related subs.
# 0.09 [2007/08/12] modified error strings/error template
# 0.08 [2007/08/10] got rid of 'plugin' from member variables because 'sb' includes it
# 0.07 [2007/07/20] removed _setup_application, content and header to simplify the class
# 0.06 [2007/07/15] ported _error_template from Serene Bach 2.*
# 0.05 [2007/07/14] changed load_template and removed save_template
# 0.04 [2007/07/08] changed error to use new sb::Template
# 0.03 [2007/02/14] changed error
# 0.02 [2006/07/23] added load_template and save_template
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/09] created
# --------------------------------------
# // defines / static variables
sub ERROR_TITLE          (){ 'error' };
sub ERROR_STRING         (){ "%s\n<code class=\"error\">%s</code>" };
sub DEFAULT_CODE         (){ 'utf8' };
sub DEFAULT_RESOURCE_DIR (){ 'lib/resource/' }
sub DEFAULT_STATIC_DIR   (){ 'resource/' };
sub DEFAULT_STYLE        (){ 'style.css' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'time'   => sb::Time->new,
		'conf'   => undef,
		'lang'   => undef,
		'cgi'    => undef,
		'apache' => undef,
		@_
	);
	$self->generate_accessor('time','conf','lang','cgi','apache');
	$self->cgi(sb::Interface->new('req'=>$self->apache)) if (!$self->cgi);
	$self->conf(sb->get->conf) if (!$self->conf);
	$self->lang(sb->get->lang) if (!$self->lang);
	sb::Lock->basedir($self->conf->value('dir_lock')); # initialize lock base directory
	return $self;
}
# --------------------------------------
# // public functions - declaration
sub run
{
	my $class = shift;
	my $app = $class->new(@_);
	$app->error(sprintf(ERROR_STRING,sb->string('Unknown error'),$@));
}
# --------------------------------------
# // public functions
sub output
{
	my $self = shift;
	$self->cgi->head(); # output header
	($self->apache) 
		? $self->apache->print(@_) 
		: CORE::print(@_);
	return;
}
sub error
{
	my $self = shift;
	my $message = shift;
	my $temp = sb::Template->new( &_error_template() );
	my $top = (index((caller)[1],'Admin') == -1)
		? $self->conf->value('conf_srv_base')
		: $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_admn');
	$temp->num(0);
	$temp->tag('sb_site_title'=>$sb::PRODUCT . ' | ' . ERROR_TITLE);
	$temp->tag('sb_error_title'=>sb->string('Error'));
	$temp->tag('sb_error'=>$message);
	$temp->tag('sb_site_top'=>$top);
	$self->common_parts($temp);
	$self->output($temp->output);
}
sub load_template
{
	my $self = shift;
	my %param = (
		'dir'  => $self->resource_dir,
		'file' => undef,
		@_
	);
	return if ($param{'file'} eq '');
	my $file = $param{'dir'} . $param{'file'};
	my $text = '';
	if (-r $file)
	{
		open(TEMPIN,"<$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(TEMPIN);
		while (my $line = <TEMPIN>)
		{
			$line =~ tr/\x0D\x0A//d;
			$text .= $line . "\n";
		}
		close(TEMPIN);
	}
	if (sb->get->conf->value('basic_temp_conv') and sb->get->lang->charcode ne DEFAULT_CODE)
	{
		$text = sb->get->lang->convert($text,sb->get->lang->charcode,DEFAULT_CODE);
	}
	return $text;
}
# --------------------------------------
# // public functions - parsing path
sub hostname
{
	my $self = shift;
	return (split('\.',$self->cgi->host))[0];
}
sub path_info
{
	my $self = shift;
	my $path = $self->cgi->path_info;
	$path =~ s!^/!!; # remove the first letter '/' from the path
	return $path;
}
sub base_path
{
	my $self = shift;
	my $path = $self->cgi->current_path;
	$path =~ s!(.*/)(.*)$!$1!;
	return $path;
}
# --------------------------------------
# // public functions - template parts
sub common_parts
{
	my $self = shift;
	my $temp  = shift;
	return if (!$temp or !$self);
	$temp->num(0);
	$temp->tag('sb_site_css'=>$self->style_path);
	$temp->tag('sb_site_template'=>$self->static_path);
	$temp->tag('sb_site_encoding'=>sb->get->lang->charset);
	$temp->tag('sb_site_lang'=>sb->get->lang->language);
	$temp->tag('sb_buildno'=>$sb::BUILDNO);
	$temp->tag('sb_version'=>$sb::VERSION);
	$temp->tag('sb_product_name'=>$sb::PRODUCT);
	$temp->tag('sb_webpage'=>$sb::WEBPAGE);
	return;
}
sub resource_dir
{
	my $self = shift;
	return sb->get->conf->value('resource_dir') || DEFAULT_RESOURCE_DIR;
}
sub static_path
{
	my $self = shift;
	return sb->get->conf->value('srv_temp') || DEFAULT_STATIC_DIR;
}
sub style_path
{
	my $self = shift;
	return sb->get->conf->value('site_css') || sb->get->conf->value('srv_temp') . DEFAULT_STYLE;
}
# --------------------------------------
# // private functions
sub _error_template
{ # default error template
	return <<'__ERROR_TEMPLATE__';
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="{sb_site_lang}">
<head>
<meta http-equiv="Content-Type" content="text/html; charset={sb_site_encoding}" />
<meta name="generator" content="{sb_product_name}" />
<title>{sb_site_title}</title>
<link rel="stylesheet" href="{sb_site_css}" type="text/css" title="default" media="screen,tv" />
</head>
<body>
<div class="head">
<h1 id="top"><a href="{sb_webpage}"><img src="{sb_site_template}image/blank.gif" alt="{sb_product_name}" width="120" height="60" title="{sb_product_name}" /></a></h1>
</div>
<div class="error">
<h2 class="error">{sb_error_title}</h2>
<p class="msg">{sb_error}</p>
<p class="msg"><a href="{sb_site_top}">return to top</a></p>
</div>
<address>Powered by <a href="{sb_webpage}">{sb_product_name}</a> {sb_version}</address>
<address>Copyright &copy; 2007 SimpleBoxes/SerendipityNZ Ltd., All rights reserved.</address>
</body>
</html>
__ERROR_TEMPLATE__
}
1;
__END__
