# sb::Language
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Language;
use strict;
use sb::InitParser ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.13';
# 0.13 [2008/12/08] modified load to load string resources correctly
# 0.12 [2008/11/24] fixed typo in charset / modified initialize
# 0.11 [2008/10/30] added mailcode
# 0.10 [2007/08/04] changed construct and setup
# 0.09 [2007/08/01] added deprecated functions for backward compatibility
# 0.08 [2007/07/17] changed load to use InitParser
# 0.07 [2007/07/16] changed return to make them consistent
# 0.06 [2006/04/06] changed load to use Spica::InitLoader
# 0.05 [2006/04/06] changed string to handle null string
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2005/11/15] added construct/setup to construct correctly
# 0.02 [2005/11/08] added %aCharsets
# 0.01 [2005/10/25] refactored
# 0.00 [2005/01/17] generated
# --------------------------------------
# // defines / static variables
my %aCharsets = (
	'iso-8859-1'  => 'ascii',
	'euc-jp'      => 'euc',
	'shift_jis'   => 'sjis',
	'iso-2022-jp' => 'jis',
	'utf-8'       => 'utf8',
	'utf-16'      => 'utf16',
);
# --------------------------------------
# // override base class
sub initialize
{ # sb::Language works as a factory of language modules.
	my $self = shift;
	my $lang = shift;
	my $class = ref($self);
	if ( $lang )
	{
		my $language = $class->module_name($lang);
		return $language->construct('lang'=>$lang) if ($class->load_module($lang));
	}
	return $class->construct;
}
# --------------------------------------
# // public functions - initializer
sub construct
{
	my $class = shift;
	my $self = bless {
		'lang'     => 'en',
		'charset'  => 'ASCII',
		'charcode' => 'ascii',
		'strings'  => {},
		@_
	},$class;
	return $self->setup();
}
sub setup
{
	my $self = shift;
	return $self->SUPER::initialize(
		@_,
		'strings' => {
			'week_en'     => ['Sun','Mon','Tue','Wed','Thu','Fri','Sat'],
			'week_enlong' => ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'],
			'week_fr'     => ['dim','lun','mar','mer','jeu','ven','sam'],
			'week_frlong' => ['dimanche','lundi','mardi','mercredi','jeudi','vendredi','samedi'],
			'week_ja'     => ['&#x65E5;','&#x6708;','&#x706B;','&#x6C34;','&#x6728;','&#x91D1;','&#x571F;'],
			'week_jalong' => [
				'&#x65E5;&#x66DC;&#x65E5;','&#x6708;&#x66DC;&#x65E5;','&#x706B;&#x66DC;&#x65E5;','&#x6C34;&#x66DC;&#x65E5;',
				'&#x6728;&#x66DC;&#x65E5;','&#x91D1;&#x66DC;&#x65E5;','&#x571F;&#x66DC;&#x65E5;'],
			'month_en'     => ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'],
			'month_enlong' => [
				'January','February','March','April','May','June',
				'July','August','September','October','November','December'],
			'month_fr'     => [
				'jan','f&eacute;v','mar','avr','mai','juin',
				'juil','ao&ucirc;','sep','oct','nov','d&eacute;c'],
			'month_frlong' => [
				'janvier','f&eacute;vrier','mars','avril','mai','juin',
				'juillet','ao&ucirc;t','septembre','octobre','novembre','d&eacute;cembre'],
		}
	);
}
# --------------------------------------
# // public functions - accessors
sub language
{
	my $self = shift;
	return $self->{'lang'};
}
sub charset
{
	my $self = shift;
	$self->{'charset'} = shift if (@_);
	return $self->{'charset'};
}
sub charcode
{
	my $self = shift;
	$self->{'charcode'} = shift if (@_);
	return $self->{'charcode'};
}
sub mailcode
{
	my $self = shift;
	return $self->charcode();
}
sub string
{
	my $self = shift;
	my $key  = shift;
	return if (! defined($key) or $key eq '');
	$self->{'strings'}->{$key} = shift if (@_);
	return $key if (! defined($self->{'strings'}->{$key}));
	return $key if ( $self->{'strings'}->{$key} eq '' );
	return $self->{'strings'}->{$key};
}
sub load
{
	my $self = shift;
	my $file = shift;
	sb::InitParser->new('file'=>$file)->load($self->{'strings'},{'strict'=>1});
}
# --------------------------------------
# // public functions - common
sub convert
{
	my $self = shift;
	my ($text,$outcode,$in_code) = @_;
	return $text;
}
sub checkcode
{
	my $self = shift;
	my $text = shift;
	return $self->charcode;
}
sub charcode_of
{
	my $self = shift;
	my $charset = lc( shift() );
	return ($aCharsets{$charset}) ? $aCharsets{$charset} : $charset;
}
sub charset_of
{
	my $self = shift;
	my $charcode = lc( shift() );
	my %code = reverse(%aCharsets);
	return ($code{$charcode}) ? $code{$charcode} : $charcode;
}
# --------------------------------------
# // public functions - deprecated
sub code
{ # the same as language, so we should use language instead of this
	language(@_);
}
sub init
{ # it always returns true
	return 1;
}
sub strings
{
	my $self = shift;
	return $self->{'strings'};
}
sub code_for_charset
{ # the same as charcode_of, so we should use charcode_of instead of this
	charcode_of(@_);
}
sub mailtext
{ # the same as convert
	convert(@_);
}
sub holiday
{ # it has been moved to Local
	return {};
}
1;
__END__
